/*!
 *****************************************************************************
 * @file:
 * @brief:
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2013-2018 Analog Devices, Inc. All Rights Reserved.

This software is proprietary to Analog Devices, Inc. and its licensors.
By using this software you agree to the terms of the associated
Analog Devices Software License Agreement.
 ******************************************************************************/

#include "ADuCM410.h"
#include "WdtLib.h"
#include "common.h"
#include "DioLib.h"
#include "ClkLib.h"
#include "PlaLib.h"
#include "GptLib.h"

extern void delay(uint32_t t);
extern void pla_test_init(void);
extern void test_scan_and(void);
extern void test_scan_or(void);
extern void test_scan_nor(void);
extern void test_scan_not(void);
extern void test_scan_xor(void);
extern void test_scan_nand(void);
extern void test_scan_exnor(void);   
extern void test_flipflop(void);

int main (void)
{

    WdtGo(false);
    ClkSetup(&gClkSetup);
    SystemInit();
    INIT();
    SafetyWait();

    DEBUG_MESSAGE("Test Started");
    
    pla_test_init();
    test_scan_and();
    test_scan_or();
    test_scan_nor();
    test_scan_not();
    test_scan_xor();
    test_scan_nand();
    test_scan_exnor();

    test_flipflop();

    while(1)
    {
     
    }
}

void pla_test_init()
{
  
  //PLA_OUT_B
  PLA_OUT_A(ELEM1,INA_PLA_DIN);
  PLA_OUT_A(ELEM2,INA_PLA_DIN);
  
  PLA_OUT_A(ELEM17,BLOCK1_INA_B0_ELEMx);  //elem17 = elem1
  PLA_OUT_A(ELEM18,BLOCK1_INA_B0_ELEMx);  //elem18 = elem2

}

#define ASSERT_OUTPUT_BLOCK0(expect)  { uint32_t out = pADI_PLA->PLA_DOUT0;\
                            if(out != (expect)){\
                            DEBUG_MESSAGE("TEST FAILED at line %d, expect 0x%x was 0x%x",__LINE__, (expect),out);\
                            }}
#define ASSERT_OUTPUT_BLOCK1(expect)  {\
                            if(pADI_PLA->PLA_DOUT1 != (expect)){\
                            DEBUG_MESSAGE("TEST FAILED at line %d, expect 0x%x was 0x%x",__LINE__, (expect),pADI_PLA->PLA_DOUT1);\
                            }}                            


#define  scan_and_pattern(elem,output_bit)      \
{       \
  PLA_AND(elem,INA_ELEM2,INB_ELEM1);     \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0(0);      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0(1<<2);   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(1<<output_bit)); \
  PLA_LOGIC_0(elem);   \
  PLA_AND(elem,INA_ELEM2_NOT,INB_ELEM1);\
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0(0);      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(1<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0(1<<2);   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(0<<output_bit)); \
  PLA_LOGIC_0(elem);   \
  PLA_AND(elem,INA_ELEM2,INB_ELEM1_NOT);\
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0(0);      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(0<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0((1<<2)|(1<<output_bit));   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(0<<output_bit)); \
  PLA_LOGIC_0(elem);   \
}      
#define  scan_and_pattern_block1(elem,output_bit)      \
{       \
  PLA_AND(elem,BLOCK1_INA_ELEM2,BLOCK1_INB_ELEM1);     \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK1(0);      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK1((1<<1)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK1(1<<2);   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK1((1u<<1)|(1<<2)|(1<<output_bit)); \
  PLA_LOGIC_0(elem);   \
  PLA_AND(elem,BLOCK1_INA_ELEM2_NOT,BLOCK1_INB_ELEM1);\
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK1(0);      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK1((1<<1)|(1<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK1(1<<2);   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK1((1u<<1)|(1<<2)|(0<<output_bit)); \
  PLA_LOGIC_0(elem);   \
  PLA_AND(elem,BLOCK1_INA_ELEM2,BLOCK1_INB_ELEM1_NOT);\
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK1(0);      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK1((1<<1)|(0<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK1((1<<2)|(1<<output_bit));   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK1((1u<<1)|(1<<2)|(0<<output_bit)); \
  PLA_LOGIC_0(elem);   \
}      

void test_scan_and()
{   
    
  scan_and_pattern(ELEM0,0);
  scan_and_pattern(ELEM3,3);
  scan_and_pattern(ELEM4,4);
  scan_and_pattern(ELEM5,5);
  scan_and_pattern(ELEM6,6);
  scan_and_pattern(ELEM7,7);
  scan_and_pattern(ELEM8,8);
  scan_and_pattern(ELEM9,9);
  scan_and_pattern(ELEM10,10);
  scan_and_pattern(ELEM11,11);
  scan_and_pattern(ELEM12,12);
  scan_and_pattern(ELEM13,13);
  scan_and_pattern(ELEM14,14);
  scan_and_pattern(ELEM15,15);
  
  
  scan_and_pattern_block1(ELEM16,0);
  scan_and_pattern_block1(ELEM19,3);
  scan_and_pattern_block1(ELEM20,4);
  scan_and_pattern_block1(ELEM21,5);
  scan_and_pattern_block1(ELEM22,6);
  scan_and_pattern_block1(ELEM23,7);
  scan_and_pattern_block1(ELEM24,8);
  scan_and_pattern_block1(ELEM25,9);
  scan_and_pattern_block1(ELEM26,10);
  scan_and_pattern_block1(ELEM27,11);
  scan_and_pattern_block1(ELEM28,12);
  scan_and_pattern_block1(ELEM29,13);
  scan_and_pattern_block1(ELEM30,14);
  scan_and_pattern_block1(ELEM31,15);
}

#define  scan_or_pattern(elem,output_bit)      \
{       \
  PLA_OR(elem,INA_ELEM2,INB_ELEM1);    \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0(0);      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(1<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0((1<<2)|(1<<output_bit));   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(1<<output_bit)); \
  PLA_LOGIC_0(elem);   \
  PLA_OR(elem,INA_ELEM2_NOT,INB_ELEM1);    \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0(1<<output_bit);      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(1<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0((1<<2)|(0<<output_bit));   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(1<<output_bit)); \
  PLA_LOGIC_0(elem);   \
  PLA_OR(elem,INA_ELEM2,INB_ELEM1_NOT);    \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0(1<<output_bit);      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(0<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0((1<<2)|(1<<output_bit));   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(1<<output_bit)); \
  PLA_LOGIC_0(elem);   \
}       

void test_scan_or()
{ 
  scan_or_pattern(ELEM0,0);
  scan_or_pattern(ELEM3,3);
  scan_or_pattern(ELEM4,4);
  scan_or_pattern(ELEM5,5);
  scan_or_pattern(ELEM6,6);
  scan_or_pattern(ELEM7,7);
  scan_or_pattern(ELEM8,8);
  scan_or_pattern(ELEM9,9);
  scan_or_pattern(ELEM10,10);
  scan_or_pattern(ELEM11,11);
  scan_or_pattern(ELEM12,12);
  scan_or_pattern(ELEM13,13);
  scan_or_pattern(ELEM14,14);
  scan_or_pattern(ELEM15,15);
}

#define  scan_nor_pattern(elem,output_bit)      \
{       \
  PLA_NOR(elem,INA_ELEM2,INB_ELEM1);    \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0((1<<output_bit));      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(0<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0((1<<2)|(0<<output_bit));   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(0<<output_bit)); \
  PLA_LOGIC_0(elem);   \
}       

void test_scan_nor()
{
  scan_nor_pattern(ELEM0,0);
  scan_nor_pattern(ELEM3,3);
  scan_nor_pattern(ELEM4,4);
  scan_nor_pattern(ELEM5,5);
  scan_nor_pattern(ELEM6,6);
  scan_nor_pattern(ELEM7,7);
  scan_nor_pattern(ELEM8,8);
  scan_nor_pattern(ELEM9,9);
  scan_nor_pattern(ELEM10,10);
  scan_nor_pattern(ELEM11,11);
  scan_nor_pattern(ELEM12,12);
  scan_nor_pattern(ELEM13,13);
  scan_nor_pattern(ELEM14,14);
  scan_nor_pattern(ELEM15,15);
}

#define  scan_not_pattern(elem,output_bit)      \
{       \
  PLA_NOT(elem,INA_ELEM2);    \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0((1<<output_bit));      \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0((1<<2)|(0<<output_bit)); \
  PLA_LOGIC_0(elem);    \
  PLA_NOT(elem,INB_ELEM1);    \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0((1<<output_bit));      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(0<<output_bit)); \
  PLA_LOGIC_0(elem);   \
}       

void test_scan_not()
{   
  scan_not_pattern(ELEM0,0);
  scan_not_pattern(ELEM3,3);
  scan_not_pattern(ELEM4,4);
  scan_not_pattern(ELEM5,5);
  scan_not_pattern(ELEM6,6);
  scan_not_pattern(ELEM7,7);
  scan_not_pattern(ELEM8,8);
  scan_not_pattern(ELEM9,9);
  scan_not_pattern(ELEM10,10);
  scan_not_pattern(ELEM11,11);
  scan_not_pattern(ELEM12,12);
  scan_not_pattern(ELEM13,13);
  scan_not_pattern(ELEM14,14);
  scan_not_pattern(ELEM15,15);
}

#define  scan_xor_pattern(elem,output_bit)      \
{       \
  PLA_XOR(elem,INA_ELEM2,INB_ELEM1);    \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0((0<<output_bit));      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(1<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0((1<<2)|(1<<output_bit));   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(0<<output_bit)); \
  PLA_LOGIC_0(elem);   \
}       

void test_scan_xor()
{   
  scan_xor_pattern(ELEM0,0);
  scan_xor_pattern(ELEM3,3);
  scan_xor_pattern(ELEM4,4);
  scan_xor_pattern(ELEM5,5);
  scan_xor_pattern(ELEM6,6);
  scan_xor_pattern(ELEM7,7);
  scan_xor_pattern(ELEM8,8);
  scan_xor_pattern(ELEM9,9);
  scan_xor_pattern(ELEM10,10);
  scan_xor_pattern(ELEM11,11);
  scan_xor_pattern(ELEM12,12);
  scan_xor_pattern(ELEM13,13);
  scan_xor_pattern(ELEM14,14);
  scan_xor_pattern(ELEM15,15);
}

#define  scan_nand_pattern(elem,output_bit)      \
{       \
  PLA_NAND(elem,INA_ELEM2,INB_ELEM1);    \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0((1<<output_bit));      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(1<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0((1<<2)|(1<<output_bit));   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(0<<output_bit)); \
  PLA_LOGIC_0(elem);   \
}       

void test_scan_nand()
{   
  scan_nand_pattern(ELEM0,0);
  scan_nand_pattern(ELEM3,3);
  scan_nand_pattern(ELEM4,4);
  scan_nand_pattern(ELEM5,5);
  scan_nand_pattern(ELEM6,6);
  scan_nand_pattern(ELEM7,7);
  scan_nand_pattern(ELEM8,8);
  scan_nand_pattern(ELEM9,9);
  scan_nand_pattern(ELEM10,10);
  scan_nand_pattern(ELEM11,11);
  scan_nand_pattern(ELEM12,12);
  scan_nand_pattern(ELEM13,13);
  scan_nand_pattern(ELEM14,14);
  scan_nand_pattern(ELEM15,15);
}

#define  scan_exnor_pattern(elem,output_bit)      \
{       \
  PLA_EXNOR(elem,INA_ELEM2,INB_ELEM1);    \
  pADI_PLA->PLA_DIN0 = 0;       \
  ASSERT_OUTPUT_BLOCK0((1<<output_bit));      \
  pADI_PLA->PLA_DIN0 = (1<<1);  \
  ASSERT_OUTPUT_BLOCK0((1<<1)|(0<<output_bit)); \
  pADI_PLA->PLA_DIN0 = (1<<2);  \
  ASSERT_OUTPUT_BLOCK0((1<<2)|(0<<output_bit));   \
  pADI_PLA->PLA_DIN0 = (1<<1)|(1<<2);   \
  ASSERT_OUTPUT_BLOCK0((1u<<1)|(1<<2)|(1<<output_bit)); \
  PLA_LOGIC_0(elem);   \
}       

void test_scan_exnor()
{   
  scan_nand_pattern(ELEM0,0);
  scan_nand_pattern(ELEM3,3);
  scan_nand_pattern(ELEM4,4);
  scan_nand_pattern(ELEM5,5);
  scan_nand_pattern(ELEM6,6);
  scan_nand_pattern(ELEM7,7);
  scan_nand_pattern(ELEM8,8);
  scan_nand_pattern(ELEM9,9);
  scan_nand_pattern(ELEM10,10);
  scan_nand_pattern(ELEM11,11);
  scan_nand_pattern(ELEM12,12);
  scan_nand_pattern(ELEM13,13);
  scan_nand_pattern(ELEM14,14);
  scan_nand_pattern(ELEM15,15);
}

void test_flipflop()
{
  DioCfgPin(pADI_GPIO0,PIN4,P0_4_PLA2_OUT);
  DioCfgPin(pADI_GPIO0,PIN5,P0_5_PLA3_OUT);
  DioCfgPin(pADI_GPIO0,PIN6,P0_6_PLA4_OUT);
  DioCfgPin(pADI_GPIO0,PIN7,P0_7_PLA5_OUT);
  DioDsPin(pADI_GPIO0,PIN4,3);
  DioDsPin(pADI_GPIO0,PIN5,3);
  DioDsPin(pADI_GPIO0,PIN6,3);
  DioDsPin(pADI_GPIO0,PIN7,3);

  PLA_OUT_A(ELEM2,INA_PLA_DIN);
  PLA_OUT_A(ELEM3,INA_PLA_DIN);
  
  BLOCK0_ALWAYS_AT_POSEDGE(PLA_CLK_T0)
  {
  PLA_BEGIN
    PLA_AND(ELEM4,INA_ELEM2,INB_ELEM3);
    PLA_OR(ELEM5,INA_ELEM2,INB_ELEM3);
  PLA_END
  }
  
  GptSetup(pADI_GPT0,&gGpt0Setup);
  GptGo(pADI_GPT0);
  NVIC_EnableIRQ(GPT0_IRQn);
  
  
  //check wave form on P0.4, P0.5, P0.6, P0.7
  pADI_PLA->PLA_DIN0 = 0; 
  delay(180*1000);      //more than 1ms
  pADI_PLA->PLA_DIN0 = (1<<2);
  delay(140*2000);      //more than 1ms
  pADI_PLA->PLA_DIN0 = (1<<3);
  delay(120*1000);      //more than 1ms
  pADI_PLA->PLA_DIN0 = (1<<3)|(1<<2);
  delay(320*1000);      //more than 1ms
  pADI_PLA->PLA_DIN0 = 0; 
}

//void test_flipflop_block1()
//{
//  DioCfgPin(pADI_GPIO0,PIN4,P0_4_PLA2_OUT);
//  DioCfgPin(pADI_GPIO0,PIN5,P0_5_PLA3_OUT);
//  DioCfgPin(pADI_GPIO0,PIN6,P0_6_PLA4_OUT);
//  DioCfgPin(pADI_GPIO0,PIN7,P0_7_PLA5_OUT);
//  DioDsPin(pADI_GPIO0,PIN4,3);
//  DioDsPin(pADI_GPIO0,PIN5,3);
//  DioDsPin(pADI_GPIO0,PIN6,3);
//  DioDsPin(pADI_GPIO0,PIN7,3);
//
//  PLA_OUT_A(ELEM2,INA_PLA_DIN);
//  PLA_OUT_A(ELEM3,INA_PLA_DIN);
//  
//  BLOCK0_ALWAYS_AT_POSEDGE(PLA_CLK_T0)
//  {
//  PLA_BEGIN
//    PLA_AND(ELEM4,INA_ELEM2,INB_ELEM3);
//    PLA_OR(ELEM5,INA_ELEM2,INB_ELEM3);
//  PLA_END
//  }
//  
//  GptSetup(pADI_GPT0,&gGpt0Setup);
//  GptGo(pADI_GPT0);
//  NVIC_EnableIRQ(GPT0_IRQn);
//  
//  
//  //check wave form on P0.4, P0.5, P0.6, P0.7
//  pADI_PLA->PLA_DIN0 = 0; 
//  delay(180*1000);      //more than 1ms
//  pADI_PLA->PLA_DIN0 = (1<<2);
//  delay(140*2000);      //more than 1ms
//  pADI_PLA->PLA_DIN0 = (1<<3);
//  delay(120*1000);      //more than 1ms
//  pADI_PLA->PLA_DIN0 = (1<<3)|(1<<2);
//  delay(320*1000);      //more than 1ms
//  pADI_PLA->PLA_DIN0 = 0; 
//}


void delay(uint32_t t)
{
   uint32_t i;
   for(i=0;i<t;i++);
}

void GP_Tmr0_Int_Handler()
{
   DEBUG_PIN1_TOGGLE();
}





